package android.exemple.TD4_1;

import java.util.ArrayList;

import android.app.Activity;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.IBinder;
import android.telephony.SmsManager;
import android.telephony.SmsMessage;
import android.util.Log;
import android.widget.Toast;

/**
 * 
 * @author fcamps
 * 
 * Ce service est un répondeur pour SMS
 * 
 * Ouvrir 2 emulateurs (port 5554 et 5556 par exemple) et lancer un "telnet localhost 5554" sur le premier
 * puis simuler l'envoie d'un sms depuis le deuxieme émulateur : “sms send +5556 hello”
 * à la réception du SMS le deuxième émulateur (5556) envoie une réponse par SMS
 */
public class ResponserService extends Service
{
	//The Action fired by the Android-System when a SMS was received.
	private static final String RECEIVED_ACTION = "android.provider.Telephony.SMS_RECEIVED";
	private static final String SENT_ACTION="SENT_SMS";
	private static final String DELIVERED_ACTION="DELIVERED_SMS";

	String requester;
	String reply="";  


	@Override
	public void onCreate()
	{
		super.onCreate();

		// The receiver will be called with any broadcast Intent that matches filter 
		registerReceiver(sentSMS, new IntentFilter(SENT_ACTION));
		//registerReceiver(deliverReceiver, new IntentFilter(DELIVERED_ACTION));
		registerReceiver(receiverSMS, new IntentFilter(RECEIVED_ACTION));
	}


	/**
	 * Cette classe permet de lancer une action à l'émission d'un SMS
	 */
	BroadcastReceiver sentSMS = new BroadcastReceiver()
	{
		@Override public void onReceive(Context c, Intent in) 
		{
			switch(getResultCode())
			{

			case Activity.RESULT_OK:
				//sent SMS message successfully;
				smsSent();
				break;
			default:
				//sent SMS message failed
				smsFailed();
				break;
			}
		}
	};

	/**
	 * 
	 */
	public void smsSent()
	{
		Toast.makeText(this, "SMS sent", Toast.LENGTH_SHORT).show();
	}

	/**
	 * 
	 */
	public void smsFailed()
	{
		Toast.makeText(this, "SMS sent failed", Toast.LENGTH_SHORT).show();
	}

	/**
	 * 
	 */
	public void smsDelivered()
	{
		Toast.makeText(this, "SMS delivered", Toast.LENGTH_SHORT).show();
	}

	/**
	 * 
	 */
	BroadcastReceiver deliverReceiver = new BroadcastReceiver()
	{
		@Override public void onReceive(Context c, Intent in)
		{
			//SMS delivered actions
			smsDelivered();
		}    
	};

	/**
	 * 
	 * @param f
	 */
	public void requestReceived(String f)
	{
		Log.v("ResponserService","In requestReceived --> requester = "+ requester);

		// n° telephone appelant
		requester=f;
	}

	/**
	 * Classe qui exécute une action à la réception d'un SMS
	 * 
	 */
	BroadcastReceiver receiverSMS = new BroadcastReceiver()
	{
		String str="";

		@Override
		public void onReceive(Context c, Intent in)
		{
			Log.v("ResponserService","On Receive");
			reply="";

			if(in.getAction().equals(RECEIVED_ACTION))
			{
				Log.v("ResponserService","On SMS RECEIVE");

				Bundle bundle = in.getExtras();

				if(bundle!=null)
				{
					Object[] pdus = (Object[])bundle.get("pdus");
					SmsMessage[] messages = new SmsMessage[pdus.length];

					for(int i = 0; i<pdus.length; i++)
					{
						Log.v("ResponserService","FOUND MESSAGE");

						messages[i] = SmsMessage.createFromPdu((byte[])pdus[i]);               

						str += messages[i].getMessageBody().toString();
						str += "\n";  
						
						// éviter le bouclage
						if(str.startsWith("REPONDEUR"))
							return;
					}
					// traitement à la réception d'un SMS
					//if(str.equals("hello\n"))
					//if(str.indexOf("I am") > 0)
																 
					if(!str.startsWith("REPONDEUR"))
					{
						System.out.println("OK1");
						
						for(SmsMessage message: messages)
						{
							requestReceived(message.getOriginatingAddress());
						}

						smsDelivered();

						respond();
					}
				}
			}
		}
	};


	@Override
	public void onStart(Intent intent, int startId) 
	{
		
		super.onStart(intent, startId);
	}

	/**
	 * 
	 * Cette méthode renvoie un SMS en fonction du contenu d'un SMS
	 * 
	 */
	public void respond()
	{
		// TODO modifier cette methode 
		// en fonction d'un SMS alors on envoie un autre SMS
		
		Log.v("ResponserService","Responing to "+requester);

		String reply = "REPONDEUR: Thank you for your message. I am busy now. I will call you later";
		SmsManager sms = SmsManager.getDefault();

		Intent sentIn = new Intent(SENT_ACTION);
		PendingIntent sentPIn = PendingIntent.getBroadcast(this,0,sentIn,0);

		Intent deliverIn = new Intent(DELIVERED_ACTION);
		PendingIntent deliverPIn = PendingIntent.getBroadcast(this,0,deliverIn,0);

		ArrayList<String> Msgs = sms.divideMessage(reply);
		ArrayList<PendingIntent> sentIns = new ArrayList<PendingIntent>();
		ArrayList<PendingIntent> deliverIns = new ArrayList<PendingIntent>();

		for(int i=0; i< Msgs.size(); i++)
		{
			sentIns.add(sentPIn);
			deliverIns.add(deliverPIn);
		}

		sms.sendMultipartTextMessage(requester, null, Msgs, sentIns, deliverIns);
	}


	@Override
	public void onDestroy() 
	{
		
		super.onDestroy();
		unregisterReceiver(sentSMS);
		unregisterReceiver(receiverSMS);

	}
	@Override
	public IBinder onBind(Intent arg0) 
	{
		
		return null;
	}

}
